@@ -1,3 +1,4 @@
+import enum
 import os
 import platform
 import weakref
@@ -28,21 +29,22 @@ except NameError:
     str_types = (str,)
 
 
-def get_icon_file(name):
+def get_icon_file(name: str) -> str:
     """
-    Returns the full path to the icon represented by name.
+    Returns the absolute path to the icon filename provided.
 
     Parameters
     ----------
     name : str
-        The filename to load the file path.
+        The filename of the icon, relative to ``pydm.icons``.
 
     Returns
     -------
     str
+        The absolute path to the icon file.
     """
     base_path = os.path.dirname(os.path.realpath(__file__))
-    icon_path = os.path.join(base_path, "icons", "terminator.png")
+    icon_path = os.path.join(base_path, "icons", name)
     return icon_path
 
 
@@ -139,8 +141,6 @@ class PyDMPrimitiveWidget(object):
         'Opacity': ['set_opacity', float]
     }
 
-    designer_icon = QIcon()
-
     def __init__(self, **kwargs):
         self.app = QApplication.instance()
         self._rules = None
@@ -150,6 +150,11 @@ class PyDMPrimitiveWidget(object):
             # and not at the Designer
             self.installEventFilter(self)
 
+    @staticmethod
+    def get_designer_icon():
+        """Icon for usage in Qt designer."""
+        return QIcon()
+
     def eventFilter(self, obj, event):
         """
         EventFilter to redirect "middle click" to :meth:`.show_address_tooltip`
@@ -322,6 +327,14 @@ class PyDMPrimitiveWidget(object):
         return None
 
 
+class AlarmLimit(str, enum.Enum):
+    """ An enum for holding values corresponding to the EPICS alarm limits """
+    HIHI = "HIHI"
+    HIGH = "HIGH"
+    LOW = "LOW"
+    LOLO = "LOLO"
+
+
 class TextFormatter(object):
 
     default_precision_from_pv = True
@@ -588,6 +601,10 @@ class PyDMWidget(PyDMPrimitiveWidget):
         self._upper_ctrl_limit = None
         self._lower_ctrl_limit = None
 
+        self.upper_alarm_limit = None
+        self.lower_alarm_limit = None
+        self.upper_warning_limit = None
+        self.lower_warning_limit = None
         self.enum_strings = None
 
         self.value = None
@@ -767,6 +784,26 @@ class PyDMWidget(PyDMPrimitiveWidget):
         else:
             self._lower_ctrl_limit = new_limit
 
+    def alarm_limit_changed(self, which: AlarmLimit, new_limit: float) -> None:
+        """
+        Callback invoked when the channel receives new alarm limit values.
+
+        Parameters
+        ----------
+        which : AlarmLimit
+            Which alarm limit was changed. "HIHI", "HIGH", "LOW", "LOLO"
+        new_limit : float
+            New value for the alarm limit
+        """
+        if which is AlarmLimit.HIHI:
+            self.upper_alarm_limit = new_limit
+        elif which is AlarmLimit.HIGH:
+            self.upper_warning_limit = new_limit
+        elif which is AlarmLimit.LOW:
+            self.lower_warning_limit = new_limit
+        elif which is AlarmLimit.LOLO:
+            self.lower_alarm_limit = new_limit
+
     @Slot(bool)
     def connectionStateChanged(self, connected):
         """
@@ -823,6 +860,7 @@ class PyDMWidget(PyDMPrimitiveWidget):
         """
         self.enum_strings_changed(new_enum_strings)
 
+    @Slot(int)
     @Slot(float)
     def upperCtrlLimitChanged(self, new_limit):
         """
@@ -836,6 +874,7 @@ class PyDMWidget(PyDMPrimitiveWidget):
         """
         self.ctrl_limit_changed("UPPER", new_limit)
 
+    @Slot(int)
     @Slot(float)
     def lowerCtrlLimitChanged(self, new_limit):
         """
@@ -849,6 +888,58 @@ class PyDMWidget(PyDMPrimitiveWidget):
         """
         self.ctrl_limit_changed("LOWER", new_limit)
 
+    @Slot(int)
+    @Slot(float)
+    def upper_alarm_limit_changed(self, new_limit: float):
+        """
+        PyQT slot for changes to the HIHI alarm limit of a PV
+
+        Parameters
+        ----------
+        new_limit : float
+           The new value for the HIHI limit
+        """
+        self.alarm_limit_changed(AlarmLimit.HIHI, new_limit)
+
+    @Slot(int)
+    @Slot(float)
+    def lower_alarm_limit_changed(self, new_limit: float):
+        """
+        PyQT slot for changes to the LOLO alarm limit of a PV
+
+        Parameters
+        ----------
+        new_limit : float
+           The new value for the LOLO limit
+        """
+        self.alarm_limit_changed(AlarmLimit.LOLO, new_limit)
+
+    @Slot(int)
+    @Slot(float)
+    def upper_warning_limit_changed(self, new_limit: float):
+        """
+        PyQT slot for changes to the HIGH alarm limit of a PV
+
+        Parameters
+        ----------
+        new_limit : float
+           The new value for the HIGH limit
+        """
+        self.alarm_limit_changed(AlarmLimit.HIGH, new_limit)
+
+    @Slot(int)
+    @Slot(float)
+    def lower_warning_limit_changed(self, new_limit: float):
+        """
+        PyQT slot for changes to the LOW alarm limit of a PV
+
+        Parameters
+        ----------
+        new_limit : float
+           The new value for the LOW limit
+        """
+        self.alarm_limit_changed(AlarmLimit.LOW, new_limit)
+
     @Slot()
     def force_redraw(self):
         """
@@ -992,6 +1083,10 @@ class PyDMWidget(PyDMPrimitiveWidget):
                                   prec_slot=None,
                                   upper_ctrl_limit_slot=self.upperCtrlLimitChanged,
                                   lower_ctrl_limit_slot=self.lowerCtrlLimitChanged,
+                                  upper_alarm_limit_slot=self.upper_alarm_limit_changed,
+                                  lower_alarm_limit_slot=self.lower_alarm_limit_changed,
+                                  upper_warning_limit_slot=self.upper_warning_limit_changed,
+                                  lower_warning_limit_slot=self.lower_warning_limit_changed,
                                   value_signal=None,
                                   write_access_slot=None)
             # Load writeable channels if our widget requires them. These should
