@@ -1,10 +1,11 @@
 import time
 import json
 from collections import OrderedDict
-from pyqtgraph import ViewBox, AxisItem
+from typing import Optional
+from pyqtgraph import BarGraphItem, ViewBox, AxisItem
 import numpy as np
 from qtpy.QtGui import QColor
-from qtpy.QtCore import Signal, Slot, Property, QTimer
+from qtpy.QtCore import Signal, Slot, Property, QTimer, Q_ENUMS
 from .baseplot import BasePlot, BasePlotCurveItem
 from .channel import PyDMChannel
 from .. utilities import remove_protocol
@@ -19,7 +20,13 @@ DEFAULT_X_MIN = -30
 DEFAULT_Y_MIN = 0
 
 DEFAULT_TIME_SPAN = 5.0
-DEFAULT_UPDATE_INTERVAL = 100
+DEFAULT_UPDATE_INTERVAL = 1000  # Plot update rate for fixed rate mode in milliseconds
+
+
+class updateMode(object):
+    """ updateMode as new type for plot update """
+    OnValueChange = 1
+    AtFixedRate = 2
 
 
 class TimePlotCurveItem(BasePlotCurveItem):
@@ -38,6 +45,9 @@ class TimePlotCurveItem(BasePlotCurveItem):
         If True, the x-axis shows timestamps as ticks, and those timestamps
         scroll to the left as time progresses.  If False, the x-axis tick marks
         show time relative to the current time.
+    plot_style: str, optional
+        Currently one of either 'Line' or 'Bar'. Determines how data points for this
+        curve will be plotted. Defaults to a line based plot if not set
     color : QColor, optional
         The color used to draw the curve line and the symbols.
     lineStyle: int, optional
@@ -52,7 +62,7 @@ class TimePlotCurveItem(BasePlotCurveItem):
     """
     _channels = ('channel',)
 
-    def __init__(self, channel_address=None, plot_by_timestamps=True, **kws):
+    def __init__(self, channel_address=None, plot_by_timestamps=True, plot_style='Line', **kws):
         """
         Parameters
         ----------
@@ -74,8 +84,10 @@ class TimePlotCurveItem(BasePlotCurveItem):
         # Keep the x-axis moving with latest timestamps as ticks
         self._plot_by_timestamps = plot_by_timestamps
 
+        self.plot_style = plot_style
+
         self._bufferSize = MINIMUM_BUFFER_SIZE
-        self._update_mode = PyDMTimePlot.SynchronousMode
+        self._update_mode = PyDMTimePlot.OnValueChange
 
         self._min_y_value = None
         self._max_y_value = None
@@ -89,7 +101,8 @@ class TimePlotCurveItem(BasePlotCurveItem):
         super(TimePlotCurveItem, self).__init__(**kws)
 
     def to_dict(self):
-        dic_ = OrderedDict([("channel", self.address), ])
+        dic_ = OrderedDict([("channel", self.address),
+                            ("plot_style", self.plot_style)])
         dic_.update(super(TimePlotCurveItem, self).to_dict())
         return dic_
 
@@ -172,7 +185,7 @@ class TimePlotCurveItem(BasePlotCurveItem):
         """
         self.update_min_max_y_values(new_value)
 
-        if self._update_mode == PyDMTimePlot.SynchronousMode:
+        if self._update_mode == PyDMTimePlot.OnValueChange:
             self.data_buffer = np.roll(self.data_buffer, -1)
             # The first array row is to record timestamps, when a new value arrives.
             self.data_buffer[0, self._bufferSize - 1] = time.time()
@@ -182,7 +195,7 @@ class TimePlotCurveItem(BasePlotCurveItem):
             if self.points_accumulated < self._bufferSize:
                 self.points_accumulated += 1
             self.data_changed.emit()
-        elif self._update_mode == PyDMTimePlot.AsynchronousMode:
+        elif self._update_mode == PyDMTimePlot.AtFixedRated:
             self.latest_value = new_value
 
     @Slot()
@@ -192,7 +205,7 @@ class TimePlotCurveItem(BasePlotCurveItem):
         buffer, together with the timestamp when this happens. Also increments
         the accumulated point counter.
         """
-        if self._update_mode != PyDMTimePlot.AsynchronousMode:
+        if self._update_mode != PyDMTimePlot.AtFixedRated:
             return
         self.data_buffer = np.roll(self.data_buffer, -1)
         self.data_buffer[0, self._bufferSize - 1] = time.time()
@@ -244,7 +257,7 @@ class TimePlotCurveItem(BasePlotCurveItem):
             self.initialize_buffer()
 
     @Slot()
-    def redrawCurve(self):
+    def redrawCurve(self, min_x: Optional[float] = None, max_x: Optional[float] = None):
         """
         Redraw the curve with the new data.
 
@@ -253,6 +266,13 @@ class TimePlotCurveItem(BasePlotCurveItem):
         On the other hand, if plot by relative time, take the time diff from
         the starting time of the curve, and plot the data to the time diff
         position on the x-axis.
+
+        Parameters
+        ----------
+        min_x: float, optional
+            The minimum timestamp to render when plotting as a bar graph.
+        max_x: float, optional
+            The maximum timestamp to render when plotting as a bar graph.
         """
         try:
             x = self.data_buffer[0, -self.points_accumulated:].astype(np.float)
@@ -261,21 +281,59 @@ class TimePlotCurveItem(BasePlotCurveItem):
             if not self._plot_by_timestamps:
                 x -= time.time()
 
-            self.setData(y=y, x=x)
+            if self.plot_style is None or self.plot_style == 'Line':
+                self.setData(y=y, x=x)
+            elif self.plot_style == 'Bar':
+                # In cases where the buffer size is large, we don't want to render 10,000+ bars on every update
+                # if only a fraction of those are actually visible. These 2 indices represent the visible range
+                # of the plot, and we will only render bars within that range.
+                min_index = np.searchsorted(x, min_x)
+                max_index = np.searchsorted(x, max_x) + 1
+                self._setBarGraphItem(x=x[min_index:max_index], y=y[min_index:max_index])
         except (ZeroDivisionError, OverflowError):
             # Solve an issue with pyqtgraph and initial downsampling
             pass
 
+    def _setBarGraphItem(self, x, y):
+        """ Set the plots points to render as bars. No need to call this directly as it will automatically
+            be handled by redrawCurve() """
+        if self.points_accumulated == 0 or len(x) == 0 or len(y) == 0:
+            return
+
+        brushes = np.array([self.color] * len(x))
+
+        if self.threshold_color is not None:
+            if self.upper_threshold is not None:
+                brushes[np.argwhere(y > self.upper_threshold)] = self.threshold_color
+            if self.lower_threshold is not None:
+                brushes[np.argwhere(y < self.lower_threshold)] = self.threshold_color
+
+        self.bar_graph_item.setOpts(x=x, height=y, brushes=brushes)
+
     def setUpdatesAsynchronously(self, value):
-        if value is True:
-            self._update_mode = PyDMTimePlot.AsynchronousMode
+        """
+        Check if value is from updatesAsynchronously(bool) or updateMode(int)
+        """
+        if type(value)==int and value == updateMode.AtFixedRate or type(value)==bool and value is True:
+            self._update_mode = PyDMTimePlot.AtFixedRate
         else:
-            self._update_mode = PyDMTimePlot.SynchronousMode
+            self._update_mode = PyDMTimePlot.OnValueChange
+        self.initialize_buffer()
+        
+    def resetUpdatesAsynchronously(self):
+        self._update_mode = PyDMTimePlot.OnValueChange
         self.initialize_buffer()
 
-    def resetUpdatesAsynchronously(self):
-        self._update_mode = PyDMTimePlot.SynchronousMode
-        self.initialize_buffer()
+    def min_x(self):
+        """
+        Provide the the oldest valid timestamp from the data buffer.
+
+        Returns
+        -------
+        float
+            The timestamp of the most recent data point recorded into the data buffer.
+        """
+        return self.data_buffer[0, -self.points_accumulated]
 
     def max_x(self):
         """
@@ -293,9 +351,9 @@ class TimePlotCurveItem(BasePlotCurveItem):
         return [self.channel]
 
 
-class PyDMTimePlot(BasePlot):
+class PyDMTimePlot(BasePlot,updateMode):
     """
-    PyDMWaveformPlot is a widget to plot one or more waveforms.
+    PyDMTimePlot is a widget to plot one or more channels vs. time.
 
     Each curve can plot either a Y-axis waveform vs. its indices,
     or a Y-axis waveform against an X-axis waveform.
@@ -313,13 +371,19 @@ class PyDMTimePlot(BasePlot):
     background: optional
         The background color for the plot.  Accepts any arguments that
         pyqtgraph.mkColor will accept.
+    bottom_axis: AxisItem, optional
+        Will set the bottom axis of this plot to the input axis. If not set, will default
+        to either a TimeAxisItem if plot_by_timestamps is true, or a regular AxisItem otherwise
     """
-    SynchronousMode = 1
-    AsynchronousMode = 2
+    OnValueChange = 1
+    AtFixedRated = 2
+
+    Q_ENUMS(updateMode)
+    updateMode = updateMode
 
     plot_redrawn_signal = Signal(TimePlotCurveItem)
 
-    def __init__(self, parent=None, init_y_channels=[], plot_by_timestamps=True, background='default'):
+    def __init__(self, parent=None, init_y_channels=[], plot_by_timestamps=True, background='default', bottom_axis=None):
         """
         Parameters
         ----------
@@ -336,17 +400,19 @@ class PyDMTimePlot(BasePlot):
             The background color for the plot.  Accepts any arguments that
             pyqtgraph.mkColor will accept.
         """
+        self._updateMode = updateMode.OnValueChange
         self._plot_by_timestamps = plot_by_timestamps
 
-        self._left_axis = AxisItem("left")
-        if plot_by_timestamps:
+        if bottom_axis is not None:
+            self._bottom_axis = bottom_axis
+        elif plot_by_timestamps:
             self._bottom_axis = TimeAxisItem('bottom')
         else:
             self.starting_epoch_time = time.time()
             self._bottom_axis = AxisItem('bottom')
 
         super(PyDMTimePlot, self).__init__(parent=parent, background=background,
-                                           axisItems={"bottom": self._bottom_axis, "left": self._left_axis})
+                                           axisItems={"bottom": self._bottom_axis})
 
         # Removing the downsampling while PR 763 is not merged at pyqtgraph
         # Reference: https://github.com/pyqtgraph/pyqtgraph/pull/763
@@ -366,7 +432,7 @@ class PyDMTimePlot(BasePlot):
 
         self.update_timer = QTimer(self)
         self.update_timer.setInterval(self._update_interval)
-        self._update_mode = PyDMTimePlot.SynchronousMode
+        self._update_mode = PyDMTimePlot.OnValueChange
         self._needs_redraw = True
 
         self.labels = {
@@ -389,9 +455,10 @@ class PyDMTimePlot(BasePlot):
         # This function gets called by PyDMTimePlot's designer plugin.
         self.redraw_timer.setSingleShot(True)
 
-    def addYChannel(self, y_channel=None, name=None, color=None,
-                    lineStyle=None, lineWidth=None, symbol=None,
-                    symbolSize=None, yAxisName=None, useArchiveData=False):
+    def addYChannel(self, y_channel=None, plot_style=None, name=None, color=None,
+                    lineStyle=None, lineWidth=None, symbol=None, symbolSize=None,
+                    barWidth=None, upperThreshold=None, lowerThreshold=None, thresholdColor=None,
+                    yAxisName=None, useArchiveData=False):
         """
         Adds a new curve to the current plot
 
@@ -399,6 +466,8 @@ class PyDMTimePlot(BasePlot):
         ----------
         y_channel : str
             The PV address
+        plot_style : str, optional
+            The style in which to render the data, either 'Line' or 'Bar'
         name : str
             The name of the curve (usually made the same as the PV address)
         color : QColor
@@ -412,6 +481,14 @@ class PyDMTimePlot(BasePlot):
             triangle, star, etc.
         symbolSize : int
             How big the symbols should be
+        barWidth: float, optional
+            Width of any bars drawn on the plot
+        upperThreshold: float, optional
+            Bars that are above this value will be drawn in the threshold color
+        lowerThreshold: float, optional
+            Bars that are below this value will be drawn in the threshold color
+        thresholdColor: QColor, optional
+            Color to draw bars that exceed either threshold
         yAxisName : str
             The name of the y axis to associate with this curve. Will be created if it
             doesn't yet exist
@@ -431,21 +508,31 @@ class PyDMTimePlot(BasePlot):
             plot_opts['lineWidth'] = lineWidth
 
         # Add curve
-        new_curve = self.createCurveItem(y_channel, self._plot_by_timestamps, name, color=color,
-                                         yAxisName=yAxisName, useArchiveData=useArchiveData, **plot_opts)
-        new_curve.setUpdatesAsynchronously(self.updatesAsynchronously)
+        new_curve = self.createCurveItem(y_channel=y_channel, plot_by_timestamps=self._plot_by_timestamps,
+                                         plot_style=plot_style, name=name, color=color, yAxisName=yAxisName,
+                                         useArchiveData=useArchiveData, **plot_opts)
+        new_curve.setUpdatesAsynchronously(self.updateMode)
         new_curve.setBufferSize(self._bufferSize)
 
         self.update_timer.timeout.connect(new_curve.asyncUpdate)
+        if plot_style == 'Bar':
+            if barWidth is None:
+                barWidth = 1.0  # Can't use default since it can be explicitly set to None and avoided
+            new_curve.bar_graph_item = BarGraphItem(x=[], height=[], width=barWidth, brush=color)
+            new_curve.setBarGraphInfo(barWidth, upperThreshold, lowerThreshold, thresholdColor)
         self.addCurve(new_curve, curve_color=color, y_axis_name=yAxisName)
+        if new_curve.bar_graph_item is not None:
+            # Must happen after addCurve() so that the view box has been created
+            new_curve.getViewBox().addItem(new_curve.bar_graph_item)
 
         new_curve.data_changed.connect(self.set_needs_redraw)
         self.redraw_timer.start()
 
         return new_curve
 
-    def createCurveItem(self, y_channel, plot_by_timestamps, name, color, yAxisName, useArchiveData, **plot_opts):
-        return TimePlotCurveItem(y_channel, plot_by_timestamps=plot_by_timestamps,
+    def createCurveItem(self, y_channel, plot_by_timestamps, plot_style, name,
+                        color, yAxisName, useArchiveData, **plot_opts):
+        return TimePlotCurveItem(y_channel, plot_by_timestamps=plot_by_timestamps, plot_style=plot_style,
                                  name=name, color=color, yAxisName=yAxisName, **plot_opts)
 
     def removeYChannel(self, curve):
@@ -489,9 +576,11 @@ class PyDMTimePlot(BasePlot):
             return
 
         self.updateXAxis()
-
+        # The minimum and maximum x-axis timestamps visible to the user
+        min_x = self.plotItem.getViewBox().state['viewRange'][0][0]
+        max_x = self.plotItem.getViewBox().state['viewRange'][0][1]
         for curve in self._curves:
-            curve.redrawCurve()
+            curve.redrawCurve(min_x=min_x, max_x=max_x)
             self.plot_redrawn_signal.emit(curve)
         self._needs_redraw = False
 
@@ -509,7 +598,7 @@ class PyDMTimePlot(BasePlot):
             return
 
         if self._plot_by_timestamps:
-            if self._update_mode == PyDMTimePlot.SynchronousMode:
+            if self._update_mode == PyDMTimePlot.OnValueChange:
                 maxrange = max([curve.max_x() for curve in self._curves])
             else:
                 maxrange = time.time()
@@ -558,14 +647,23 @@ class PyDMTimePlot(BasePlot):
         self.clearCurves()
         for d in new_list:
             color = d.get('color')
+            thresholdColor = d.get('thresholdColor')
             if color:
                 color = QColor(color)
+            if thresholdColor:
+                thresholdColor = QColor(thresholdColor)
             self.addYChannel(d['channel'],
-                             name=d.get('name'), color=color,
+                             plot_style=d.get('plot_style'),
+                             name=d.get('name'),
+                             color=color,
                              lineStyle=d.get('lineStyle'),
                              lineWidth=d.get('lineWidth'),
                              symbol=d.get('symbol'),
                              symbolSize=d.get('symbolSize'),
+                             barWidth=d.get('barWidth'),
+                             upperThreshold=d.get('upperThreshold'),
+                             lowerThreshold=d.get('lowerThreshold'),
+                             thresholdColor=thresholdColor,
                              yAxisName=d.get('yAxisName'))
 
     curves = Property("QStringList", getCurves, setCurves, designable=False)
@@ -602,10 +700,12 @@ class PyDMTimePlot(BasePlot):
         curve = self.findCurve(curve.channel)
         if curve:
             self.removeYChannel(curve)
-            self.addYChannel(y_channel=curve.address, color=curve.color,
+            self.addYChannel(y_channel=curve.address, plot_style=curve.plot_style, color=curve.color,
                              name=curve.address, lineStyle=curve.lineStyle,
                              lineWidth=curve.lineWidth, symbol=curve.symbol,
-                             symbolSize=curve.symbolSize, yAxisName=curve.y_axis_name)
+                             symbolSize=curve.symbolSize, barWidth=curve.bar_width,
+                             upperThreshold=curve.upper_threshold, lowerThreshold=curve.lower_threshold,
+                             thresholdColor=curve.threshold_color, yAxisName=curve.y_axis_name)
 
     def addLegendItem(self, item, pv_name, force_show_legend=False):
         """
@@ -679,20 +779,23 @@ class PyDMTimePlot(BasePlot):
     bufferSize = Property("int", getBufferSize, setBufferSize, resetBufferSize)
 
     def getUpdatesAsynchronously(self):
-        return self._update_mode == PyDMTimePlot.AsynchronousMode
+        return self._update_mode == PyDMTimePlot.AtFixedRated
 
-    def setUpdatesAsynchronously(self, value):
+    def setUpdatesAsynchronously(self, value): 
         for curve in self._curves:
             curve.setUpdatesAsynchronously(value)
-        if value is True:
-            self._update_mode = PyDMTimePlot.AsynchronousMode
+        """
+        Check if value is from updatesAsynchronously(bool) or updateMode(int)
+        """
+        if type(value)==int and value == updateMode.AtFixedRate or type(value)==bool and value is True: 
+            self._update_mode = PyDMTimePlot.AtFixedRate
             self.update_timer.start()
         else:
-            self._update_mode = PyDMTimePlot.SynchronousMode
+            self._update_mode = PyDMTimePlot.OnValueChange
             self.update_timer.stop()
 
     def resetUpdatesAsynchronously(self):
-        self._update_mode = PyDMTimePlot.SynchronousMode
+        self._update_mode = PyDMTimePlot.OnValueChange
         self.update_timer.stop()
         for curve in self._curves:
             curve.resetUpdatesAsynchronously()
@@ -700,7 +803,31 @@ class PyDMTimePlot(BasePlot):
     updatesAsynchronously = Property("bool",
                                      getUpdatesAsynchronously,
                                      setUpdatesAsynchronously,
-                                     resetUpdatesAsynchronously)
+                                     resetUpdatesAsynchronously, designable=False)
+
+    @Property(updateMode)
+    def updateMode(self):
+        """
+        The updateMode to be used as property to set plot update mode.
+
+        Returns
+        -------
+        updateMode
+        """
+        return self._updateMode
+
+    @updateMode.setter
+    def updateMode(self, new_type):
+        """
+        The updateMode to be used as property to set plot update mode.
+
+        Parameters
+        ----------
+        new_type : updateMode
+        """
+        if new_type != self._updateMode:
+            self._updateMode = new_type
+            self.setUpdatesAsynchronously(self._updateMode)
 
     def getTimeSpan(self):
         """
@@ -792,8 +919,7 @@ class PyDMTimePlot(BasePlot):
     def getAutoRangeX(self):
         if self._plot_by_timestamps:
             return False
-        else:
-            super(PyDMTimePlot, self).getAutoRangeX()
+        return super(PyDMTimePlot, self).getAutoRangeX()
 
     def setAutoRangeX(self, value):
         if self._plot_by_timestamps:
